/*
 * SBMS_S25FL127S.c
 *
 *  Created on: Jan 7, 2016
 *      Author: dacus
 */

#include "SBMS_S25FL127S.h"
#include "delay.h"

#define ERASE         0xDC    // erase one sector
#define ERSP          0x75    // erase suspend
#define ERRS          0x7A    // erase resume
#define PGSP          0x85    // erase suspend
#define PGRS          0x8A    // erase resume
#define WRITE         0x12    // write data
#define READ          0x13    // read data
#define WREN          0x06    // set write enable
#define WRDI          0x04    // reset write enable
#define RDSR          0x05    // status register1
#define RDSR2         0x07    // status register2
#define DUMMY         0x00    // Dummy byte

//   RDSR status

#define SRWD             0x80
#define P_ERR            0x40
#define E_ERR            0x20
#define BP2              0x10
#define BP1              0x08
#define BP0              0x04
#define WEL              0x02
#define WIP              0x01


#define S25FL127S_CS_SET			GPIO_SetBits(S25FL127S_CS_PORT, S25FL127S_CS_PIN)
#define S25FL127S_CS_RESET			GPIO_ResetBits(S25FL127S_CS_PORT, S25FL127S_CS_PIN)

void S25FL127S_Init()
{
  TM_GPIO_Init(S25FL127S_CS_PORT, S25FL127S_CS_PIN, TM_GPIO_Mode_OUT, TM_GPIO_OType_PP, TM_GPIO_PuPd_NOPULL, TM_GPIO_Speed_Medium);
  S25FL127S_CS_SET;//CS high
  TM_SPI_Init(S25FL127S_SPI, S25FL127S_SPI_PINS);
}


void WriteEnd(void)
{
  uint8_t s = 0;
  S25FL127S_CS_RESET;
  SPI_Send(S25FL127S_SPI, RDSR);
  do { s = SPI_Send(S25FL127S_SPI, DUMMY);}
  while ((s & WIP) == SET);
  S25FL127S_CS_SET;
}

void WriteDisable(void)
{
  S25FL127S_CS_RESET;
  SPI_Send(S25FL127S_SPI, WRDI);
  S25FL127S_CS_SET;
  WriteEnd();
}

void WriteEnable(void)
{
  S25FL127S_CS_RESET;
  SPI_Send(S25FL127S_SPI, WREN);
  S25FL127S_CS_SET;
  WriteEnd();
}




ReadFlash(uint32_t Addr,uint8_t* Fdata,uint16_t Nr)
{
  uint16_t i;
  S25FL127S_CS_RESET;
  SPI_Send(S25FL127S_SPI, ERSP);
  S25FL127S_CS_SET;

  S25FL127S_CS_RESET;
  SPI_Send(S25FL127S_SPI, READ);
  SPI_Send(S25FL127S_SPI, ((Addr & 0xFF000000) >> 24));
  SPI_Send(S25FL127S_SPI, ((Addr & 0x00FF0000) >> 16));
  SPI_Send(S25FL127S_SPI, ((Addr & 0x0000FF00) >> 8));
  SPI_Send(S25FL127S_SPI,  (Addr & 0x000000FF));
  for (i=0;i<Nr;i++){ Fdata[i] = SPI_Send(S25FL127S_SPI, DUMMY);}
  S25FL127S_CS_SET;
  S25FL127S_CS_RESET;
  SPI_Send(S25FL127S_SPI, ERRS);
  S25FL127S_CS_SET;
}



void WriteFlash(uint8_t Val, uint32_t Addr)
{

	WriteEnable();
	S25FL127S_CS_RESET;
	SPI_Send(S25FL127S_SPI, WRITE);
	SPI_Send(S25FL127S_SPI, ((Addr & 0xFF000000) >> 24));
	SPI_Send(S25FL127S_SPI, ((Addr & 0x00FF0000) >> 16));
	SPI_Send(S25FL127S_SPI, ((Addr & 0x0000FF00) >> 8));
	SPI_Send(S25FL127S_SPI,  (Addr & 0x000000FF));
	SPI_Send(S25FL127S_SPI, Val);
	S25FL127S_CS_SET;
	WriteEnd();
	WriteDisable();
}



void EraseSector(uint32_t Addr)
{
	WriteEnable();
	S25FL127S_CS_RESET;
	SPI_Send(S25FL127S_SPI, ERASE);
	SPI_Send(S25FL127S_SPI, ((Addr & 0xFF000000) >> 24));
	SPI_Send(S25FL127S_SPI, ((Addr & 0x00FF0000) >> 16));
	SPI_Send(S25FL127S_SPI, ((Addr & 0x0000FF00) >> 8));
	SPI_Send(S25FL127S_SPI,  (Addr & 0x000000FF));
	S25FL127S_CS_SET;
}
